#  多端阅读消息数同步（同步会话状态）



> 提示：:同步会话已读状态，进入会话或者退出会话的时候可以调用这个方法，其他端在收到已读消息回执（onReceiveReadReceip）的代理方法中通知会话列表更新



#### 1. 在进入和退出会话页面的时候调用下方接口

```dart
/// 同步会话阅读状态
///
/// [conversationType] 会话类型，参见枚举 [RCConversationType]
///
/// [targetId] 会话 id
///
/// [timestamp] 该会话已阅读的最后一条消息的发送时间戳
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
///
static void syncConversationReadStatus(int conversationType, String targetId,
    int timestamp, Function(int code) finished) async {
  Map map = {
    "conversationType": conversationType,
    "targetId": targetId,
    "timestamp": timestamp
  };

  Map result = await _channel.invokeMethod(
      RCMethodKey.SyncConversationReadStatus, map);
  int code = result["code"];
  if (finished != null) {
    finished(code);
  }
}


// test：
for(int i=0;i<messageDataSource.length;i++) {
  Message message = messageDataSource[i];
    if (message.messageDirection == RCMessageDirection.Receive) {
      RongIMClient.syncConversationReadStatus(this.conversationType, this.targetId, message.sentTime, (int code){
        if (code == 0) {
          print('syncConversationReadStatusSuccess');
        } else {
          print('syncConversationReadStatusFailed:code = + $code');
        }
      });
      break;
    }
}
```



#### 2. 在收到已读消息回执的时候通知会话列表更新

```dart
prefix.RongIMClient.onReceiveReadReceipt = (Map map) {
	// 发送通知让会话列表更新
  print("object onReceiveReadReceipt " + map.toString());
};
```





[iOS 多端阅读消息数同步](https://docs.rongcloud.cn/im/imlib/ios/functions/syncreadstatus/)

[Android 多端阅读消息数同步](https://docs.rongcloud.cn/im/imlib/android/functions/syncreadstatus/)

