# 群组已读回执


> 群组已读回执逻辑：UserA 在群组 GroupA 中发送了一条文本消息，然后针对这条文本消息想实现群组已读回执。
1. UserA 在发送文本消息成功后，针对这条消息发送已读回执请求（sendReadReceiptRequest）
2. 在 GroupA 中的其他成员就会收到已读回执请求（onMessageReceiptRequest）
3. 在 GroupA 中的 UserB 读了这条文本消息，就需要发送已读回执响应（sendReadReceiptResponse）
4. UserA 就会收到已读回执响应（onMessageReceiptResponse），此时需要更新消息的已读人数



#### 1、群组消息请求回执

对于需要阅读之后收到阅读回执的消息，可以调用这个接口来发送阅读回执请求:

```dart
	/// 请求消息阅读回执
  /// 
  /// [message] 要求阅读回执的消息
  /// 
  /// [timestamp] 该会话已阅读的最后一条消息的发送时间戳
  /// 
  /// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
  /// 此接口只支持群组
  static void sendReadReceiptRequest(Message message, Function(int code) finished) async {
    int messageId = message.messageId;
    String messageJson = message.toString();
    Map messageMap = MessageFactory.instance.message2Map(message);
    Map map = {
      "messageId": messageId,
      "message": messageJson,
      "messageMap": messageMap
    };

    Map result = await _channel.invokeMethod(RCMethodKey.SendReadReceiptRequest,map);
    int code = result["code"];
    if(finished != null) {
      finished(code);
    }
  }
```


#### 2、已读响应

当收到阅读回执请求之后，如果用户阅读了对应的消息，可以调用此接口来发送消息阅读回执响应。

```dart
  /// 发送阅读回执
  /// 
  /// [conversationType] 会话类型，参见枚举 [RCConversationType]
  /// 
  /// [targetId] 会话 id
  /// 
  /// [messageList] 已经阅读了的消息列表
  /// 
  /// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
  /// 此接口只支持群组
  static void sendReadReceiptResponse(int conversationType,String targetId,List messageList,  Function(int code) finished) async {
    Map map = {
      "conversationType": conversationType,
      "targetId": targetId,
      "messageList": messageList
    };

    Map result = await _channel.invokeMethod(RCMethodKey.SendReadReceiptResponse,map);
    int code = result["code"];
    if(finished != null) {
      finished(code);
    }
  }
```


#### 3、收到已读回执请求的回调

当发送方发送已读回执请求时，会走这个回调函数（收到此请求后，如果用户阅读了对应的消息，需要调用 sendMessageReadReceiptResponse 接口发送已读响应）

```dart
  ///请求消息已读回执
  ///
  ///[data] 回执的内容 {messageUId=请求已读回执的消息ID, conversationType=会话类型, targetId=会话的targetId}
  ///
  ///eg:{targetId='c1Its71dc', conversationType=3, messageUId='MessageUId'}
  static Function(Map data) onMessageReceiptRequest;
```



#### 4、收到已读回执响应的回调

当收到已读回执响应时，会走这个回调函数（收到此响应后，可以从消息中获取到阅读回执消息的 messageUId 和 阅读消息用户的列表，消息的发送方即可直接知道哪些人已经阅读）


```dart
  ///消息已读回执响应（收到阅读回执响应，可以按照 messageUId 更新消息的阅读数）
  ///
  ///[data] 回执的内容 {messageUId=请求已读回执的消息ID, conversationType=会话类型, targetId=会话的targetId, userIdList=已读userId列表}
  ///
  ///eg:{targetId='c1Its71dc', conversationType=3, messageUId='MessageUId', readerList=['userId1', 'userId2']}
  static Function(Map data) onMessageReceiptResponse;
```





[iOS 群组消息阅读回执](https://docs.rongcloud.cn/im/imlib/ios/functions/readreceipt-group/)

[Android 群组消息阅读回执](https://docs.rongcloud.cn/im/imlib/android/functions/readreceipt-group/)