#  聊天室属性自定义

为指定聊天室以 Key、Value 方式自定义设置属性信息，此服务需要在开发者后台开通 IM 商用版后，开通使用。



> **应用场景：**
>
> 1、语音直播室场景，可用于做会场属性的同步，如房间内的成员属性，麦位上的人员等等。
>
> 2、狼人杀等卡牌类的游戏场景，可用于记录用户的角色，牌局状态等。



> **警告：**
>
> 每个聊天室中，最多允许设置 100 个属性信息，以 Key、Value 的方式进行存储，每个聊天室中每秒钟最多允许操作 Key、Value 100 次。 聊天室销毁后，聊天室中的自定义属性同时销毁。

#### 1. 设置属性 & 强制设置属性

##### 1. 设置属性

```dart
/// 设置聊天室自定义属性
///
/// [chatroomId] 聊天室 Id
///
/// [key] 聊天室属性名称，Key 支持大小写英文字母、数字、部分特殊符号 + = - _ 的组合方式，最大长度 128 个字符
///
/// [value] 聊天室属性对应的值，最大长度 4096 个字符
///
/// [sendNotification] 是否需要发送通知，如果发送通知，聊天室中的其他用户会接收到 RCChatroomKVNotificationMessage 通知消息，消息内容中包含操作类型(type)、属性名称(key)、属性名称对应的值(value)和自定义字段(extra)
///
/// [autoDelete] 用户掉线或退出时，是否自动删除该 Key、Value 值；自动删除时不会发送通知
///
/// [notificationExtra] 通知的自定义字段，RC:chrmKVNotiMsg 通知消息中会包含此字段，最大长度 2 kb
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void setChatRoomEntry(
    String chatroomId,
    String key,
    String value,
    bool sendNotification,
    bool autoDelete,
    String notificationExtra,
    Function(int code) finished) async {
  Map map = {
    "chatroomId": chatroomId,
    "key": key,
    "value": value,
    "sendNotification": sendNotification,
    "autoDelete": autoDelete,
    "notificationExtra": notificationExtra
  };
  int result = await _channel.invokeMethod(RCMethodKey.SetChatRoomEntry,map);
  if(finished != null) {
    finished(result);
  }
}
```



##### 2. 强制设置属性

```dart

/// 强制设置聊天室自定义属性
///
/// [chatroomId] 聊天室 Id
///
/// [key] 聊天室属性名称，Key 支持大小写英文字母、数字、部分特殊符号 + = - _ 的组合方式，最大长度 128 个字符
///
/// [value] 聊天室属性对应的值，最大长度 4096 个字符
///
/// [sendNotification] 是否需要发送通知，如果发送通知，聊天室中的其他用户会接收到 RCChatroomKVNotificationMessage 通知消息，消息内容中包含操作类型(type)、属性名称(key)、属性名称对应的值(value)和自定义字段(extra)
///
/// [autoDelete] 用户掉线或退出时，是否自动删除该 Key、Value 值；自动删除时不会发送通知
///
/// [notificationExtra] 通知的自定义字段，RC:chrmKVNotiMsg 通知消息中会包含此字段，最大长度 2 kb
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void forceSetChatRoomEntry(
    String chatroomId,
    String key,
    String value,
    bool sendNotification,
    bool autoDelete,
    String notificationExtra,
    Function(int code) finished) async {
  Map map = {
    "chatroomId": chatroomId,
    "key": key,
    "value": value,
    "sendNotification": sendNotification,
    "autoDelete": autoDelete,
    "notificationExtra": notificationExtra
  };
  int result = await _channel.invokeMethod(RCMethodKey.ForceSetChatRoomEntry,map);
  if(finished != null) {
    finished(result);
  }
}
```





#### 2. 删除属性&强制删除属性

##### 1. 删除属性

```dart
/// 删除聊天室自定义属性
///
/// [chatroomId] 聊天室 Id
///
/// [key] 聊天室属性名称
///
/// [sendNotification] 是否需要发送通知，如果发送通知，聊天室中的其他用户会接收到 RCChatroomKVNotificationMessage 通知消息，消息内容中包含操作类型(type)、属性名称(key)、属性名称对应的值(value)和自定义字段(extra)
///
/// [notificationExtra] 通知的自定义字段，RC:chrmKVNotiMsg 通知消息中会包含此字段，最大长度 2 kb
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void removeChatRoomEntry(
    String chatroomId,
    String key,
    bool sendNotification,
    String notificationExtra,
    Function(int code) finished) async {
  Map map = {
    "chatroomId": chatroomId,
    "key": key,
    "sendNotification": sendNotification,
    "notificationExtra": notificationExtra
  };
  int result = await _channel.invokeMethod(RCMethodKey.RemoveChatRoomEntry,map);
  if(finished != null) {
    finished(result);
  }
}
```



##### 2. 强制删除属性

```dart
/// 强制删除聊天室自定义属性
///
/// [chatroomId] 聊天室 Id
///
/// [key] 聊天室属性名称
///
/// [sendNotification] 是否需要发送通知，如果发送通知，聊天室中的其他用户会接收到 RCChatroomKVNotificationMessage 通知消息，消息内容中包含操作类型(type)、属性名称(key)、属性名称对应的值(value)和自定义字段(extra)
///
/// [notificationExtra] 通知的自定义字段，RC:chrmKVNotiMsg 通知消息中会包含此字段，最大长度 2 kb
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void forceRemoveChatRoomEntry(
    String chatroomId,
    String key,
    bool sendNotification,
    String notificationExtra,
    Function(int code) finished) async {
  Map map = {
    "chatroomId": chatroomId,
    "key": key,
    "sendNotification": sendNotification,
    "notificationExtra": notificationExtra
  };
  int result = await _channel.invokeMethod(RCMethodKey.ForceRemoveChatRoomEntry,map);
  if(finished != null) {
    finished(result);
  }
}
```



#### 3. 获取属性

##### 1. 获取聊天室单个属性

```dart
/// 获取聊天室单个属性
///
/// [chatroomId] 聊天室 Id
///
/// [key] 聊天室属性名称
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败，entry 为返回的 map
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void getChatRoomEntry(String chatroomId, String key,
    Function(Map entry, int code) finished) async {
  Map map = {"chatroomId": chatroomId, "key": key};

  Map result = await _channel.invokeMethod(RCMethodKey.GetChatRoomEntry, map);
  int code = result["code"];
  Map entry = result["entry"];
  if (finished != null) {
    finished(entry, code);
  }
}
```



##### 2. 获取聊天室所有属性

```dart
/// 获取聊天室所有自定义属性
///
/// [chatroomId] 聊天室 Id
///
/// [finished] 回调结果，code 为 0 代表操作成功，其他值代表失败，entry 为返回的 map
/// 此接口只支持聊天室，必须先开通聊天室属性自定义功能
static void getAllChatRoomEntries(
    String chatroomId, Function(Map entry, int code) finished) async {
  Map map = {"chatroomId": chatroomId};

  Map result =
      await _channel.invokeMethod(RCMethodKey.GetAllChatRoomEntries, map);
  int code = result["code"];
  Map entry = result["entry"];
  if (finished != null) {
    finished(entry, code);
  }
}
```



[iOS 聊天室属性自定义](https://docs.rongcloud.cn/im/imlib/ios/chatroom/#entry)

[Android 聊天室属性自定义](https://docs.rongcloud.cn/im/imlib/android/chatroom/#entry)

